/*
 	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SHARED_H
#define SHARED_H

#include "lists.h"

typedef void *(*clone_func_t)(void *so);

typedef struct shared_object_class
{
	char *id;
	void **base;
	list_destroy_t destroy;
	clone_func_t clone;
	int size;
	int offs;
	int offs2;
} shared_object_class;

DEFLIST(exported_names,
		char *extname;
		);

DEFLIST(allow_list,
		void *subject;
		);

typedef struct shared_object
{
		exported_names *extnames;
		int refcnt;
		allow_list *allowed;
		allow_list *denied;
		void *owner;
		shared_object_class *class;
} shared_object;

DEFLIST(shared_int,
		char *intname;
		void *obj;
		void *env;
		shared_object *so;
		);

int find_byref(void *obj, void *key);
int so_find(void *obj, char *name);
int si_find_exact(shared_int *si, char *name);
int si_find_match(shared_int *si, char *key);
int si_find_byso(shared_int *si, void *so);

void destroy_shared_int(shared_int *si);
void destroy_so(shared_object *so);

void export_shared(shared_object *so, char *extname);
void *find_exported(void *clsname, char *name);
shared_int *embed_shared(void *object, void *clsname, char *name);
shared_int *import_shared(char *iname, char *name, void *clsname);

shared_int *init_shared_object(void *env, void *so, void *clsname, char *intname);
shared_int *clone_shared_object(void *env, void *so, void *clsname, char *intname);

shared_int *get_si_list(void *env, void *cls);
shared_int *find_si_itself(void *env, void *cls, char *iname);
void *quiet_find_shared_int(void *env, void *cls, char *iname);
void *find_shared_int(void *env, void *cls, char *iname);

void check_si_perm(void *env, shared_int *si);
void allow_shared_int(void *env, shared_int *si, void *dest);
void deny_shared_int(void *env, shared_int *si, void *dest);

void rename_shared_int(void *env, void *cls, char *name, char *newname, int force);

void add_shared_int(void *env, shared_int *si);
void append_shared_int(void *env, shared_int *si);

void remove_shared_int(void *env, void *cls, char *name);
#endif
